#include "Bang.h"
#include "Vram.h"
#include "Sprite.h"
#include "Stage.h"
#include "Coord.h"

constexpr byte Size = 1 * CoordRate;
constexpr byte NextSprite = SpriteCount;

constexpr byte Status_None = 0x00;
constexpr byte Status_Small = 0x10;
constexpr byte Status_Large_Small = 0x20;
constexpr byte Status_Large_Large = 0x30;

constexpr byte StatusMask = 0xf0;
constexpr byte CountMask = 0x0f;

constexpr byte RangeX = WindowWidth * CoordRate - 1;
constexpr byte RangeY = WindowHeight * CoordRate;

Bang[MaxBangCount] Bangs;

void InitBangs()
{
    ptr<Bang> pBang;
    for (pBang : Bangs) {
        ptr<byte> pSprite;
        pBang->status = Status_None;
    }
}


void StartBang(byte x, byte y, bool large)
{
    ptr<Bang> pBang;
    for (pBang : Bangs) {
        if ((pBang->status & 0xf0) != Status_None) continue;

        pBang->x = x;
        pBang->y = y;
        pBang->status = large ? Status_Large_Small : Status_Small;
        return;
    }
}



static byte Show(byte x, byte y, byte sprite, ptr<byte> pChars)
{
    if (sprite < NextSprite) {
        x -= Size;
        y -= Size;
        if (x < RangeX && y < RangeY) {
            ShowSprite(sprite, x, y, pChars);
            return sprite + 1;
        }
    }
    return sprite;
}


void UpdateBangs()
{
    static const byte[] Chars = {
        0xfe, 0xfd, 0xfb, 0xf6, 0xf9, 0xfe, 0xff, 0xf8,
        0xfd, 0xf6, 0xf2, 0xf7, 0xfb, 0xf4, 0xf6, 0xfb,
        0xf9, 0xff, 0xf7, 0xf9,
    };
    byte sprite;
    ptr<Bang> pBang;
    sprite = Sprite_Bang;
    for (pBang : Bangs) {
        byte mode, count;
        mode = pBang->status & 0xf0;
        if (mode == Status_None) continue;
        count = pBang->status & 0x0f;

        if (mode == Status_Large_Large) {
            sprite = Show(pBang->x - Size, pBang->y - Size, sprite, Chars + 4);
            sprite = Show(pBang->x + Size, pBang->y - Size, sprite, Chars + 8);
            sprite = Show(pBang->x - Size, pBang->y + Size, sprite, Chars + 12);
            sprite = Show(pBang->x + Size, pBang->y + Size, sprite, Chars + 16);
        }
        else {
            sprite = Show(pBang->x, pBang->y, sprite, Chars);
        }

        ++count;
        if (count >= 4) {
            if (mode == Status_Large_Small) {
                pBang->status = Status_Large_Large;
            }
            else {
                pBang->status = Status_None;
            }
        }
        else {
            pBang->status = mode | count;
        }
    }
    while (sprite < NextSprite) {
        HideSprite(sprite);
        ++sprite;
    }
}