#include "Status.h"
#include "Vram.h"
#include "Print.h"
#include "Sprite.h"
#include "Fighter.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + VramRowSize * 0 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 3 + WindowWidth * VramStep, "HIGH");
    PrintS(Vram + VramRowSize * 4 + WindowWidth * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 7 + WindowWidth * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 8 + VramWidth - 2 * VramStep, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 1 + VramWidth - 6 * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 5 + VramWidth - 6 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 21 + (WindowWidth + 1) * VramStep;
    if (RemainCount > 1) {
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2S(pVram, FighterChars);
            pVram = Erase2(pVram);
            pVram += VramRowSize - 2;
            *pVram = i + 0x10; ++pVram;
            pVram += 1 - VramRowSize;
        }
        else {
            do {
                pVram = Put2S(pVram, FighterChars);
                ++pVram;
                --i;
            } while (i > 0);
        }
    }
    if (RemainCount < 3) {
        pVram = Erase2(pVram);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (WindowWidth - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	G
            0xf0, 0xfe, 0xf3, 0xfd, 
            0xfa, 0xf5, 0xfc, 0xfc, 
            0xf2, 0xfd, 0xf0, 0xff, 
            0xf0, 0xf2, 0xf3, 0xf3, 
            //	U
            0xfa, 0xf5, 0xf0, 0xff, 
            0xfa, 0xf5, 0xf0, 0xff, 
            0xfa, 0xf5, 0xf0, 0xff, 
            0xf0, 0xf3, 0xf3, 0xf1, 
            //	N
            0xfa, 0xfd, 0xf0, 0xff, 
            0xfa, 0xff, 0xfd, 0xff, 
            0xfa, 0xf5, 0xfb, 0xff, 
            0xf2, 0xf1, 0xf0, 0xf3, 
            //	T
            0xf2, 0xfb, 0xf7, 0xf1, 
            0xf0, 0xfa, 0xf5, 0xf0, 
            0xf0, 0xfa, 0xf5, 0xf0, 
            0xf0, 0xf2, 0xf1, 0xf0, 
            //	U
            0xfa, 0xf5, 0xf0, 0xff, 
            0xfa, 0xf5, 0xf0, 0xff, 
            0xfa, 0xf5, 0xf0, 0xff, 
            0xf0, 0xf3, 0xf3, 0xf1, 
            //	S
            0xf8, 0xf7, 0xf3, 0xfd, 
            0xf2, 0xfd, 0xfc, 0xf4, 
            0xf8, 0xf4, 0xf0, 0xff, 
            0xf0, 0xf3, 0xf3, 0xf1, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (WindowWidth - LogoLength * 4)  * VramStep / 2;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p; ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + (WindowWidth - 14) * VramStep / 2, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + (WindowWidth - 14) * VramStep / 2, "OR CR KEY");
    PrintS(Vram + VramRowSize * 23 + (WindowWidth - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     pVram = Vram;
    //     c = 0;
    //     repeat (0) {
    //         *pVram = c; ++pVram;
    //         ++c;
    //         if ((c & 15) == 0) {
    //             pVram += VramRowSize - 16;
    //         }
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
